*-----------------------------------------------------------
* Program    : Echo Client v1.2
* Written by : Chuck Kelly
* Date       : May-15-2010
* Description: For EASy68K v5.6.0 or later
* Port 48161
* Send messages to echo server and display what is returned.
*-----------------------------------------------------------

* Display string
PRINT   MACRO
    ifnc    \1,A1           ; if first parameter is not A1
    lea     \1,A1           ; get string name from macro parameter
    endc
    move.b  #14,d0
    trap    #15             ; display string at (A1) without newline
    ENDM

* Display network error
PRINT_ERROR     MACRO
    movem.l d0/a1,-(sp)
    and.l   #$0ffff,d0          ; remove extended error code
    sub.l   #1,d0               ; make error code 0-7
    mulu.w  #netErrWidth,d0     ; calculate offset to error message
    lea     netErrors,a1
    lea     (a1,d0),a1
    PRINT   A1
    movem.l (sp)+,d0/a1
    ENDM


	ORG	$1000
START:				        ; first instruction of program

    ; check EASy68K version
    move.b  #32,d0
    move.b  #4,d1
    trap    #15             ; version number returned in D1.L
    if.l d1 <lo> #$050600 then.s    ; if older than version 5.6.0
        PRINT   versionError
        bra     halt
    endi

    ; ***** create client *****
    repeat
        PRINT   protocolMnu
        move.b  #5,d0           ; get character
        trap    #15
    until.b d1 <eq> #'0' or.b d1 <eq> #'1' do.s
    sub.b   #$30,d1         ; convert '0' or '1' character to digit
    move.w  d1,protocol     ; save protocol
    
    PRINT   promptIP
    move.b  #2,d0           ; read string 
    lea     remoteIP,a1
    trap    #15             ; read IP address as dotted quad nnn.nnn.nnn.nnn

    move.b  #100,d0         ; create client
    move.w  #48161,d1       ; port 48161
    swap    d1
    move.w  protocol,d1     ; UDP or TCP
    lea     remoteIP,a2
    trap    #15
    if.l D0 <ne> #0 then.s  ; if error
        PRINT_ERROR
        bra   halt
    endi

    ; display serverIP
    PRINT   serverIPmsg
    move.l  a2,a1
    PRINT   A1              ; print IP address at (A1)
    PRINT   CRLF

    ; display clientIP
    move.b  #105,d0
    lea     localIP,a2
    trap    #15             ; get local IP
    PRINT   clientIPmsg
    move.l  a2,a1
    PRINT   A1              ; print IP address at (A1)
    PRINT   CRLF


    ; send messages to server and display what is echoed back    
loop
    lea     keybuf,a1
    move.l  #7,d0           ; check for keyboard input
    trap    #15             ; d1.b set too 1 if input pending else 0
    if.b d1 <eq> #1 then.s  ; if input pending
        move.b #2,d0        ; get input, length in D1
        trap  #15
        move.b  #102,d0     ; send data
        trap    #15
        if.l d0 <ne> #0 then.s  ; if error
            PRINT_ERROR
        endi          
    endi
    
    ; check for receive
    move.b  #103,d0         ; receive data
    lea     netbuf,a1       ; receive buffer
    lea     remoteIP,a2     ; IP address of sender
    move.l  #BUFSIZE,d1     ; max chars to read
    trap    #15
    if.l d0 <eq> #0 and.l d1 <hi> #0 then.l  ; if characters received
        ; display incomming message
        move.b  #0,d0       ; display d1 chars with CR,LF
        lea     netbuf,a1
        trap    #15
    endi
    
    ; insert short delay to reduce CPU workload
    move.b  #23,d0          ; delay
    move.l  #1,d1           ; 1/100 second
    trap    #15
    
    bra     loop     

Halt 
    MOVE.B	#9,D0
	TRAP	#15		            ; halt simulator

* Constants
CR          equ $D              ; Carriage Return
LF          equ $A              ; Line Feed
BS          equ $8              ; Backspace
BUFSIZE     equ 80


* Variables and Strings
promptIP
        dc.b    CR,LF,'Enter IP address or name of server: ',0

serverIPmsg
        dc.b    'Server IP is: ',0
clientIPmsg
        dc.b    'Client IP is: ',0

CRLF    dc.b    CR,LF,0
        
        ds.w    0           ; force even
bufIP   ds.b    20

remoteIP    ds.b  BUFSIZE   ; remote IP as string 'aaa.bbb.ccc.ddd',0 or domain name
localIP     ds.b  16        ; local IP as string 'aaa.bbb.ccc.ddd',0

            ds.w  0         ; force even
protocol    ds.w  1         ; 0 for UDP, 1 for TCP
        
        ds.w    0           ; force word alignment
netbuf  ds.b    BUFSIZE     ; network data buffer

        ds.w    0
keybuf  ds.b    BUFSIZE     ; keyboard input buffer

versionError    dc.b    'This program requires EASy68K v5.6.0 or newer.',CR,LF,0
protocolMnu     dc.b    CR,LF,'Select Protocol',CR,LF
                dc.b    '0 = UDP',CR,LF
                dc.b    '1 = TCP',CR,LF,CR,LF
                dc.b    'Choice: ',0

* All netErrors messages must be 23 characters + 1 null for total of 24
netErrWidth equ 24
netErrors
        dc.b    'General network error  ',0
        dc.b    'Network init failed    ',0
        dc.b    'Invalid socket         ',0
        dc.b    'Get host by name failed',0
        dc.b    'Bind failed            ',0
        dc.b    'Connect failed         ',0
        dc.b    'Port alreay in use     ',0
        dc.b    'Domain not found       ',0

    END	START		; last line of source







































*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
